package de.hftstuttgart.dtabackend.utils;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.nio.file.Path;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hftstuttgart.dtabackend.models.Result;
import de.hftstuttgart.dtabackend.models.ResultSummary;
import de.hftstuttgart.dtabackend.models.TestCompetencyProfile;

import java.io.FileNotFoundException;

public class CompetencyAssessmentUtil {
	private static final Logger LOG = LogManager.getLogger(CompetencyAssessmentUtil.class);
			
	public static String TEST_COMPETENCY_MANIFEST_FILE_NAME="competency-tests.mft";
	
/*	public static void main(String[] args) throws StreamReadException, DatabindException, MalformedURLException, IOException {
		List<TestCompetencyProfile> testCompetencyProfiles=readTestCompetencyProfiles(Path.of(args[0]), args[1]);
		sumTestCompetencyProfiles(testCompetencyProfiles);
		ObjectMapper objectMapper = new ObjectMapper();
        ResultSummary resultSummary = objectMapper.readValue(
            new File(Path.of(args[0]).toFile(), args[2]).toURI().toURL(),
            ResultSummary.class);
		sumSuccessfulCompetencyProfiles(testCompetencyProfiles, resultSummary);
	}
*/	
	public static float[] sumTestCompetencyProfiles(List<TestCompetencyProfile> testCompetencyProfiles) {
		float[] tcpTotalProfile=new float[TestCompetencyProfile.MAX_COMPETENCY_DIMENSIONS];
		for(TestCompetencyProfile currentProfile: testCompetencyProfiles) {
			tcpTotalProfile=TestCompetencyProfile.competencySum(tcpTotalProfile, currentProfile.competencyAssessments);
		}
		return tcpTotalProfile;
	}
	
	public static float[] sumSuccessfulCompetencyProfiles(List<TestCompetencyProfile> testCompetencyProfiles, ResultSummary resultSummary) {
		float[] sumSuccessful=new float[TestCompetencyProfile.MAX_COMPETENCY_DIMENSIONS];
		for(Result currentResult: resultSummary.results) {
			if(currentResult.state==Result.State.SUCCESS.ordinal()) {
				TestCompetencyProfile currentProfile=new TestCompetencyProfile();
				currentProfile.testPackageName=currentResult.packageName;
				currentProfile.testClassName=currentResult.className;
				currentProfile.testName=currentResult.name;
				int testIndex=testCompetencyProfiles.indexOf(currentProfile);
				sumSuccessful=TestCompetencyProfile.competencySum(sumSuccessful, testCompetencyProfiles.get(testIndex).competencyAssessments);
			}
		}
		return sumSuccessful;
	}
	
	public static List<TestCompetencyProfile> readTestCompetencyProfiles(Path testPath, String fileName) {
		List<TestCompetencyProfile> testCompetencyProfiles=new ArrayList<TestCompetencyProfile>();
		try {
			BufferedReader testCompetencyManifest=new BufferedReader(new FileReader(new File(testPath.toFile(), fileName)));
			String testEntry=testCompetencyManifest.readLine();
			while(testEntry!=null)
			{
				String[] testEntyComponents=testEntry.split(TestCompetencyProfile.COMPETENCY_SEPARATOR);
				TestCompetencyProfile currentProfile=new TestCompetencyProfile();
				currentProfile.testPackageName=testEntyComponents[0];
				currentProfile.testClassName=testEntyComponents[1];
				currentProfile.testName=testEntyComponents[2];
				for(int competencyIndex=0; competencyIndex<TestCompetencyProfile.MAX_COMPETENCY_DIMENSIONS; competencyIndex++) {
					currentProfile.competencyAssessments[competencyIndex]=Float.valueOf(testEntyComponents[competencyIndex+3]);
				}
				testCompetencyProfiles.add(currentProfile);
				testEntry=testCompetencyManifest.readLine();
			}
			testCompetencyManifest.close();
			LOG.info("Added "+testCompetencyProfiles.size()+" test competency profiles from test competency manifest. Optional agent functionality enabled.");
		} catch (FileNotFoundException e) {
			LOG.info("Test competency manifest file for agent feedback not found. Skipping optional functionality.");
			testCompetencyProfiles=null;
		} catch (IOException e) {
			LOG.info("Test competency manifest file for agent feedback unreadable. Skipping optional functionality.");
			testCompetencyProfiles=null;
		} 
		return testCompetencyProfiles;
	}

}
